/* monaco.h */

#include "emu.h"
#include "sound/samples.h"
//#include "video/generic.h"

#define kRESCUE_CAR_SPEED (1.75)
#define SCREEN_WIDTH	384 /* 12 car lengths */
#define SCREEN_HEIGHT	240 /* 15 car widths */
#define BRIDGE_YPOS	((SCREEN_HEIGHT-16)/2-8)
#define PAGE_SIZE		(14*32)
#define NUM_COMPUTER_CARS 4

/* red */
#define TEXT_GAMEOVER		2
#define TEXT_EXTENDEDPLAY	0

/* blue */
#define TEXT_DEPOSITCOIN	3
#define TEXT_START		1

enum
{
	GFX_UNKNOWN,GFX_EXPLOSION,GFX_PLAYER,GFX_SHAKE,GFX_SPINOUT1,GFX_SPINOUT2,
	GFX_SPRAY,GFX_COMPUTER,GFX_TEXT,
	GFX_TREE,GFX_GRASS,GFX_SHRUB,GFX_HOUSE,
	GFX_TUNNEL,GFX_POOL,GFX_BELT,
	GFX_RESCUE_CAR,GFX_SIGNAL,GFX_DUMMY,
	GFX_BRIDGE1,GFX_BRIDGE2
};

enum
{
	CYAN_PEN	= 0x91,
	BLACK_PEN	= 0x94,
	YELLOW_PEN	= 0x95,
	GREY_PEN	= 0x96
};

enum
{
	RESCUE_CLUT	= 0x50,
	HOUSE_CLUT	= 0x60,
	WATER_CLUT	= 0x70,
	PLAYER_CLUT	= 0x80,
	TREE_CLUT	= 0x84,
	SHRUB_CLUT	= 0x88,
	GRASS_CLUT	= 0x8c,
	CYAN_CLUT	= 0x90,
	RED_CLUT	= 0x92,
	YELLOW_CLUT	= 0x94
};

enum
{
	GREEN_CAR, YELLOW_CAR, /* slow */
	CYAN_CAR, PURPLE_CAR, /* medium */
	BLUE_CAR, /* fast */
	NUM_COMPUTER_CAR_TYPES
};

enum
{
	PAGE_SMOOTH,
	PAGE_SLIP,
	PAGE_TUNNEL,
	PAGE_GRAVEL,
	PAGE_BRIDGE
};

enum
{
	COLLISION_CRASH,
	COLLISION_POOL
};

enum monaco_mode
{
	MODE_ATTRACT,
	/*	cars zoom up the screen (max 2 on the screen at a time)
	 *	player's car is not visible
	 *	"GAME OVER" is centered near the middle-top
	 *	"DEPOSIT COIN" is centered near the bottom
	 */
	MODE_START,
	MODE_INVULNERABLE,
	MODE_NORMAL,	/* driving */
	MODE_GRAVEL,
	MODE_SLIP1,		/* skidding */
	MODE_SLIP2,
	MODE_SPINOUT1,
	MODE_SPINOUT2,
	MODE_SPINOUT3,
	MODE_SPINOUT4,
	MODE_CRASH1,
	MODE_CRASH2,
	MODE_CRASH3,
	MODE_CRASH4
};

class monaco_state : public driver_device
{
public:
	monaco_state(const machine_config &mconfig, device_type type, const char *tag)
		: driver_device(mconfig, type, tag)
		, m_samples(*this, "samples")
		, m_p_ram(*this, "ram")
		{ }

	required_device<samples_device> m_samples;
	required_shared_ptr<UINT8> m_p_ram;
	struct monaco_gfx *m_monaco_gfx;
	enum monaco_mode m_monaco_mode;
	DECLARE_READ8_MEMBER(monaco_ram_r);
	DECLARE_WRITE8_MEMBER(monaco_ram_w);
	DECLARE_DRIVER_INIT(monaco);
	INTERRUPT_GEN_MEMBER(monaco_interrupt);
	DECLARE_MACHINE_RESET(monaco);
	DECLARE_VIDEO_START(monaco);
	UINT32 monaco_word_r (UINT16 offset);
	void monaco_word_w (UINT16 offset, UINT32 data);
	int get_player_xpos( void );
	void handle_collision( int sx, int sy, int width, int height, int type );
	int read_coin();
	void update_player_speed();
	double get_player_delta( void );
	void HandlePlayer();
	void update_computer_cars( void );
	void HandleRescue( void );
	void HandlePool( void );
	void GameOver();
	void HandleEvents();
	UINT32 screen_update_monaco(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect);
	void draw_computer( bitmap_ind16 &bitmap, const rectangle clip);
	void draw_pool( bitmap_ind16 &bitmap, const rectangle clip);
	void draw_player( bitmap_ind16 &bitmap, const rectangle clip);
	void draw_strip( bitmap_ind16 &bitmap, int sy, int x0, int x1, int xpos, int pen);
	void DrawSmoothZone( bitmap_ind16 &bitmap, const rectangle clip, int xpos);
	void DrawSlipZone( bitmap_ind16 &bitmap, const rectangle clip, int xpos);
	void DrawGravelZone( bitmap_ind16 &bitmap, const rectangle clip, int xpos);
	void DrawBridgeZone( bitmap_ind16 &bitmap, const rectangle clip, int xpos);
	void DrawTunnelZone( bitmap_ind16 &bitmap, int xpos);
	void DrawTunnelWall( bitmap_ind16 &bitmap, const rectangle clip, int xpos);
	void draw_light_helper( bitmap_ind16 &bitmap, int xpos);
	void draw_wall( bitmap_ind16 &bitmap, const rectangle clip);
	void draw_light( bitmap_ind16 &bitmap);
	void draw_page( bitmap_ind16 &bitmap, const rectangle clip, int which, int xpos);
	void draw_background( bitmap_ind16 &bitmap, const rectangle clip);
	void draw_text( bitmap_ind16 &bitmap, const rectangle clip);
	void draw_leds( bitmap_ind16 &bitmap);
	void draw_signal( bitmap_ind16 &bitmap);
	int m_anim_timer;

	/* LED display */
	UINT16 m_plays;
	UINT16 m_rank;
	UINT16 m_rank_display;	// shows 0 until game ends
	UINT32 m_score;
	UINT32 m_bonus_score;
	UINT8 m_in_ext_play;
	UINT8 m_gear;
	UINT16 m_time;
	UINT8 m_lives;
	int m_bShaking;
	double m_speed;
	double m_player_ypos;
	double m_computer_speed;
	double m_xpos[NUM_COMPUTER_CARS];
	double m_ypos[NUM_COMPUTER_CARS];
	double m_dy[NUM_COMPUTER_CARS];
	double m_rescue_xpos;
	double m_pool_xpos;
	double m_scroll;
	double m_distance;
	int m_ticks;
	int m_page_current;
	int m_page_next;
	int m_page_next2;

	double m_track_bottom_inset;
	double m_track_top_inset;
	double m_track_bottom_delta;
	double m_track_top_delta;
	int m_bSignalVisible;
	int m_left_text;
	int m_right_text;
	int m_bExtendedPlay;
	int m_left_page, m_right_page;
	int m_top_inset, m_bottom_inset;
	int m_player_x, m_player_y, m_player_tile, m_player_splash;
	int m_pool_x, m_pool_y;
	int m_rescue_x, m_rescue_y, m_rescue_tile;
	int m_x[NUM_COMPUTER_CARS];
	int m_y[NUM_COMPUTER_CARS];
	int m_tile[NUM_COMPUTER_CARS];
	int m_color[NUM_COMPUTER_CARS];
	UINT32 m_led_high1;
	UINT32 m_led_high2;
	UINT32 m_led_high3;
	UINT32 m_led_high4;
	UINT32 m_led_high5;
	UINT32 m_led_score;
	UINT16 m_led_time;
	UINT16 m_led_rank;
	UINT16 m_led_plays;
	UINT16 m_led_lives;
	UINT16 m_led_gear;
	UINT16 m_led_speed;
};
